
/*                   DETECTEUR D'ORAGES
 * Pin A1:  Entrée analogique du signal de détection d'orages.
 * Pin D2:  Entrée numérique du micro.
 * Pin D3:  Option. Inverseur Simple info ou mise en sécurité.
 *          (D3 est à la masse si l'option n'est pas utilisée)
 * Pin D12: Option. Pour coupure d'une installation losqu'un coup de foudre est détecté.(Durée 20 mn)
 * Pin D13: Sortie Alarme Pin 13. Info lors d'une détection 
 *
 * Version 1.1  Aout 2017
 */


//LCD
#include <LiquidCrystal.h>
// LCD RS pin vers pin D8
// LCD Enable vers pin D9
// LCD DB4 pin vers pin D4
// LCD DB5 pin vers pin D5
// LCD DB6 pin vers pin D6
// LCD DB7 pin vers pin D7
// LCD R/W pin vers ground
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);


int Energie = 1;     // Entrée analogique signal de détection d'orages.
int Micro = 2;       // Entrée digital du micro.
int InterSecur = 3;  // Inverseur position mise en sécurité.
int CdeSecur = 12;
int Alarme = 13;     // Sortie "Alarme".

int x, Intensit, Detect, Distance;
int ValEnergie = 0;
int ValMicro = 0;
int ValSecur = 0;
boolean Mesure;
boolean Securite;

long previousMillis = 0;       // Valeur compteur lors de la dernière utilisation
long interval = 500;            // interval (millisecondes)

void setup() {
  Serial.begin (9600);
  pinMode (Alarme, OUTPUT);    // Sortie "Alarme"
  pinMode (CdeSecur, OUTPUT);  // Sortie Commande sécurité
  pinMode (Micro, INPUT);      // Entrée micro
  pinMode (InterSecur, INPUT); //
  Detect = 0;
  Mesure = false;

  //Presentation
  lcd.begin(16, 2);
  lcd.setCursor(0, 0);
  lcd.print ("      ORAGE      ");
  lcd.setCursor(0, 1);
  lcd.print (" F5MNA Version 1 ");
  delay(3000);
  lcd.setCursor(0, 1);
  lcd.print ("    EN VEILLE    ");
  Serial.println ("  Logiciel ORAGE ");
}                

void StopAlarme() {
  digitalWrite(Alarme, Detect);
}

void GoSecur() {
  digitalWrite (CdeSecur, ValSecur);
  Mesure = false;
  Serial.println ();
  Serial.println ("securite ON");
    lcd.setCursor(0, 0);
  lcd.print ("     ORAGES     ");
  lcd.setCursor(0, 1);
  lcd.print ("  Securite ON   ");
  for (int t = 0; t < 3; t++)
  {delay (10000);}                     // TEST   10 secondes * 2   =  mn
  digitalWrite (CdeSecur, 0);
  Serial.println ("securite OFF");
  lcd.print ("  Securite OFF  ");
 }

void loop() {
  ValSecur = digitalRead(InterSecur);

  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis > interval) {
    //////////////////////////////////////////////////////////////////////////////////
    if (Mesure == false) {
      ValEnergie = analogRead(Energie);
      if (ValEnergie > 959) Intensit = 0;
      if (ValEnergie < 960) Intensit = 1;
      if (ValEnergie < 900) Intensit = 2;
      if (ValEnergie < 840) Intensit = 3;
      if (ValEnergie < 780) Intensit = 4;
      if (ValEnergie < 720) Intensit = 5;
      if (ValEnergie < 660) Intensit = 6;
      if (ValEnergie < 600) Intensit = 7;
      if (ValEnergie < 540) Intensit = 8;
      if (ValEnergie < 480) Intensit = 9;
      if (ValEnergie < 420) Intensit = 10;
      if (ValEnergie < 360) Intensit = 11;
      if (ValEnergie < 300) Intensit = 12;
      if (ValEnergie < 240) Intensit = 13;
      if (ValEnergie < 180) Intensit = 14;
      if (ValEnergie < 120) Intensit = 15;
      if (ValEnergie < 60)  Intensit = 16;


      if (Intensit > 4)   // Niveau de detection.
      { Detect = 1;
        previousMillis = currentMillis;
        delay (500);///////////////////////////////////////
        Mesure = true;
        lcd.clear();
        lcd.setCursor(0, 0);
        for (int x = 0; x < (Intensit); x++) {
          lcd.print (char(255));
          Serial.print ("X");
        }
        if (ValSecur == 1) GoSecur();
        StopAlarme();

      }
    }
  }

  if (Mesure == true)
  { if (((currentMillis - previousMillis) > 15000)) { // limitation distance à 5 Km. 15" entre éclair et tonnerre.
      Mesure = false;
      Detect = 0;
      StopAlarme();
      lcd.setCursor(0, 1);
      lcd.print ("   Hors limites  ");
      Serial.println ();
      Serial.println ("Hors limites");
    }

    ValMicro = digitalRead(Micro);

    if (ValMicro == 1) {
      Serial.println ();
      Serial.print ("Distance :");
      Distance = (int ((currentMillis - previousMillis) * 0.34 ));  
      Serial.print (Distance);  
      Serial.println (" metres");
      Serial.println ();
      Serial.println ();
      lcd.setCursor(0, 1);
      lcd.print ("Distance:  ");
      lcd.print (Distance);
      lcd.print ( "m");
      Mesure = false;
      Detect = 0;
      StopAlarme();
      previousMillis = currentMillis;
    }
  }
}

